'use strict'
interface ColorLinearOption extends Object {
  type?: String,
  x?: Number,
  y?: Number,
  x2?: Number,
  y2?: Number,
  colorStops: Array<{offset?: Number, color?: String} | Object>,
  global?: Boolean,
}
/**
 * @class 
 * @property 
 */
export default class Pie {
    public option: Object;
    public keys: Array<String | Number | unknown>;
    public values: Array<String | Number | unknown>;
    public targets: Array<String | Number | unknown>;
    public title: String;
    public legendTitle: Array<String>;
    public isHai: Boolean; // 这里就是 目标 和 实际的关系 大于？小于？变红
    protected colorRedOption: ColorLinearOption;
    protected colorYellowOption: ColorLinearOption;
    protected colorGreenOption: ColorLinearOption;
    constructor(props: any) {
      this.keys = props.keys || ['默认1', '默认2'];
      this.values = props.values || [20, 30];
      this.title = props.title || `默认标题`;
      this.targets = props.targets || [30, 20];
      this.legendTitle = props.legendTitle || ['Default-1', 'Default-2'];
      this.isHai = props.isHai || false;
      this.colorRedOption = {
        type: 'linear',
        x: 0,
        y: 0,
        x2: 0,
        y2: 1,
        colorStops: [{
            offset: 0, color: 'rgba(252, 2, 33, 1)' // 0% 处的颜色
        }, {
            offset: 1, color: 'rgba(252, 2, 33, 0.5)' // 100% 处的颜色
        }],
        global: false // 缺省为 false
      };
      this.colorYellowOption = {
        type: 'linear',
        x: 0,
        y: 0,
        x2: 0,
        y2: 1,
        colorStops: [{
            offset: 0, color: 'rgba(255, 255, 0, 1)' // 0% 处的颜色
        }, {
            offset: 1, color: 'rgba(255, 255, 0, .5)' // 100% 处的颜色
        }],
        global: false // 缺省为 false
      };
      this.colorGreenOption = {
        type: 'linear',
        x: 0,
        y: 0,
        x2: 0,
        y2: 1,
        colorStops: [{
            offset: 0, color: 'rgba(65, 218, 88, 1)' // 0% 处的颜色
        }, {
            offset: 1, color: 'rgba(65, 218, 88, 0.5)' // 100% 处的颜色
        }],
        global: false // 缺省为 false
      };

      this.option = {
        title:{
            text:`${this.title}`,
            right: 0,
            top: 0,
            left: 0,
            bottom: 0,
            textStyle:{
                color:'#000',
                fontSize: 20,
                fontWeight: 700,
            }
        },
        tooltip: {
            trigger: 'axis',
            confine: true,
            enterable: true,
            transitionDuration: 1,
            backgroundColor: 'rgba(0, 0, 0, .7)',
            // formatter: null
            textStyle: {
               color: '#fff',
            },
            axisPointer: {
                type: 'shadow',
                crossStyle: {
                    color: '#999'
                }
            }
        },
        grid: {
            top: "5%",
            bottom: "5%",
            left: "0%",
            right: "0%",
        },
        legend: {
            left: '35%',
            textStyle: {
                color: "#000",
                fontWeight: 700,
                fontSize: 16,
            },
            data: this.legendTitle,  
        },
        xAxis: [
            {
                type: 'category',
                axisLine: {
                    lineStyle: {
                        color: "rgb(0,0,0)",
                    }
                },
                data: this.keys,
                axisPointer: {
                    type: 'shadow',
                },
                axisLabel: {
                    textStyle: {
                        color: '#000',
                        fontStyle: 'normal',
                        fontWeight: 700,
                        fontSize: 16,
                    },
                },
            }
        ],
        yAxis: [
            {
                type: 'value',
                axisLabel: {
                    show: true, // is Show Y 轴
                    //formatter: '{value}--',
                    color: 'rgb(0,0,0)',
                },
                splitLine: {
                    show: true, // 基线显示 
                    lineStyle: {
                        color: "rgb(246,251,253,0.2)",
                    }
                },
            },
        ],
        series: [
            {
                name: this.legendTitle[0],
                type: 'bar',
                data: this.values,
                barWidth: '30%',
                itemStyle: {
                    normal:{
                        color: (params: any) => {
                            let value = Number(params.value);
                            let target = Number(this.targets[params.dataIndex]);
                            if(this.isHai) {
                              return value >= target?this.colorGreenOption:this.colorRedOption;
                            }else{
                              return value < target?this.colorGreenOption:this.colorRedOption;
                            }
                        },
                        label:{
                            show:true,
                            formatter: '{c}%',
                            position: 'top',
                            // position:['15%','20%'],
                            textStyle:{
                                color:'#000',
                                fontSize: 16,
                                fontWeight: 700,
                                // textShadowBlur: 3,
                                // textShadowOffsetX: 2,
                                // textShadowOffsetY: 3,
                                // textShadowColor: 'rgba(0, 0, 0, 1)',
                            }
                        }
                    },

                }
            },
            {
                name: this.legendTitle[1],
                type: 'line',
                yAxisIndex: 0,
                data: this.targets,
                itemStyle: {
                    color: {
                        type: 'linear',
                        x: 0,
                        y: 0,
                        x2: 0,
                        y2: 1,
                        colorStops: [{
                            offset: 0, color: 'rgba(16, 164, 253, 1)' // 0% 处的颜色
                        }, {
                            offset: 1, color: 'rgba(16, 164, 253, 0.5)' // 100% 处的颜色
                        }],
                        global: false // 缺省为 false
                    }
                }
            }
        ]
    }
    }
    /**
     * @param {null} unknown 可定义 
     * @returns {void} 无返回
     */
    public init(): void {

    }
    /**
     * 
     * @param prop 
     * @returns prop
     */
    public get<T>(prop: T): T {
       return prop;
    }
}
