'use strict';
import * as echarts from 'echarts';
export var PyzChartsEventType;
(function (PyzChartsEventType) {
    PyzChartsEventType["click"] = "click";
    PyzChartsEventType["dblclick"] = "dblclick";
    PyzChartsEventType["mousedown"] = "mousedown";
    PyzChartsEventType["mousemove"] = "mousemove";
    PyzChartsEventType["mouseup"] = "mouseup";
    PyzChartsEventType["mouseover"] = "mouseover";
    PyzChartsEventType["mouseout"] = "mouseout";
    PyzChartsEventType["globalout"] = "globalout";
    PyzChartsEventType["contextmenu"] = "contextmenu";
    PyzChartsEventType["resize"] = "resize";
    PyzChartsEventType["init"] = "init";
})(PyzChartsEventType || (PyzChartsEventType = {}));
export var PyzChartsEvents = [
    'click',
    'dblclick',
    'mousedown',
    'mousemove',
    'mouseup',
    'mouseover',
    'mouseout',
    'globalout',
    'contextmenu'
];
export var PyzCharts = {
    //template: `<div :id="idName" :class="{[className]: true}" class="PE-Charts"></div>`,
    name: "PyzCharts",
    props: {
        option: {
            type: Object,
            default: function () { return {}; },
        },
        watchRenderDeeper: {
            type: Object,
            default: null,
        },
        jsonMap: {
            type: Object,
            default: null,
        },
        mapName: {
            type: String,
            default: "PYZ",
        }
    },
    data: function () {
        return {
            tag: 'div',
            styled: "height:100%; width: 100%;",
            ABCD: ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J'],
            abcd: ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j'],
            className: 'PE-Charts',
            idName: 'PE-Charts-ID',
            ectp: null,
        };
    },
    created: function () {
        this.initClassName();
    },
    mounted: function () {
        var _this = this;
        this.$nextTick(function () { setTimeout(function () { _this.init(_this.$props.option); }); });
    },
    render: function (createElement) {
        return createElement(this.tag, { attrs: { id: this.idName, class: this.className, style: this.styled, } });
    },
    destroyed: function () {
        this.removeResize();
        this.$destroy();
    },
    watch: {
        option: function (val) {
            this.initECharts(val);
        },
        watchRenderDeeper: function () {
            this.$forceUpdate();
        },
    },
    methods: {
        init: function (val) {
            this.initECharts(val);
        },
        initClassName: function () {
            var _this = this;
            this.ABCD.forEach(function () {
                _this.className += "".concat(_this.ABCD[parseInt(String(Math.random() * 10))]).concat(parseInt(String(Math.random() * 10)));
                _this.idName += "".concat(_this.abcd[parseInt(String(Math.random() * 10))]).concat(parseInt(String(Math.random() * 10)));
            });
        },
        // @Params val: option interface
        initECharts: function (val) {
            if (this.ectp) {
                this.ectp.clear();
                this.ectp.setOption(val);
            }
            else {
                var ecDom = document.querySelector(".".concat(this.className));
                if (ecDom && this.$props.jsonMap) { // 防止引入生命周期 NULL
                    echarts.registerMap(this.$props.mapName, JSON.parse(JSON.stringify(this.$props.jsonMap)));
                    this.ectp = echarts.init(ecDom);
                    this.ectp.clear();
                    this.$emit(PyzChartsEventType.init, this.ectp); // 回传 ECharts 实例
                    this.ectp.setOption(val);
                    this.registryEchartsEvents(this.ectp);
                }
                else if (ecDom) {
                    this.ectp = echarts.init(ecDom);
                    this.ectp.clear();
                    this.$emit(PyzChartsEventType.init, this.ectp); // 回传 ECharts 实例
                    this.ectp.setOption(val);
                    this.registryEchartsEvents(this.ectp);
                }
            } //this.ectp.dispose();
        },
        // @Params val Events List
        registoryResize: function (target) {
            window.addEventListener(PyzChartsEventType.resize, this.rel, true);
        },
        // @Parmas Event BLO
        rel: function () {
            this.ectp.resize();
        },
        // @Remote Events
        removeResize: function () {
            window.removeEventListener(PyzChartsEventType.resize, this.rel, true);
        },
        // @Registry Events
        registryEchartsEvents: function (target) {
            var _this = this;
            if (target) {
                var defaultEvent = PyzChartsEvents;
                this.registoryResize();
                defaultEvent.forEach(function (item) {
                    target.on(item, function (params) {
                        _this.$emit(item, params);
                    });
                });
            }
        }
    },
};
